#pragma once
#include "Orbitersdk.h"
#include "UMmuSDK.h"
#include "UCGOCargoSDK.h" //UCGO 2.0 copy past this line

// Vessel Parameters
// This is where data about the vessel class can be specified, then loaded latr under a shorter name. For example,
// const double EXP_EMPTYMASS = 14770; allows me to put EXP_EMPTYMASS in place of 14770 when I specify the empty 
// mass of the vehicle later.

const double EXP_SIZE = 142.3; // mean radius in meters
const VECTOR3 EXP_CS = {7000,19000,2700}; //Shuttle-D cross section in m^2
const VECTOR3 EXP_PMI = {5000,5600,1200}; //Principal Moments of Inertia, normalized, m^2
const double EXP_EMPTYMASS = 790e6; //empty vessel mass in kg
const double VACSHD_ISP = 9000; //fuel-specific impulse in m/s
const double NMLSHD_ISP = 4200; //fuel-specific impulse in m/s
const double VACRCS_ISP = 1600; //fuel-specific impulse in m/s
const double NMLRCS_ISP = 710; //fuel-specific impulse in m/s
const double P_NML = 101.4e3;
const double rot_speed = 10.0 / 360.0;


class Enterprise :public VESSEL3
{
public:
    Enterprise (OBJHANDLE hObj, int fmodel);



	~Enterprise();

	// In this section functions to be called in the main body of the code are specified for use later. If a function placed in here is
	// never called later a "UNRESOLVED external" error will most likely pop up at compile-time. If a function is placed in the CPP but
	// not "created" here, it simply wont work.

	MESHHANDLE DVCInterior;
	void DefineAnimations();
	void clbkSetClassCaps (FILEHANDLE cfg);
	void clbkLoadStateEx (FILEHANDLE scn, void *status);
	bool clbkDrawHUD (int mode, const HUDPAINTSPEC *hps, oapi::Sketchpad *skp);
	void clbkSaveState (FILEHANDLE scn);
	void clbkPostStep (double simtt, double simdt, double mjd);
	double UpdateMass ();
	void clbkPostCreation(void);

	// Bits of code used to give the gear & payload bay references to work with instead of 0,0.1,0.2...
	// The different VC camera positions are also identified here as well.

	// This is a unique id, used to identify the ship in OrbiterSound.

	int ENT;	

	int  clbkConsumeBufferedKey (DWORD key, bool down, char *kstate);
	void clbkVisualCreated (VISHANDLE vis, int refcount);

	//UMMU 2.0 Code
	//This section contains code which is used to add support for UMMU crew, created by Dansteph.

		// UMMU 2.0 DECLARATION
	UMMUCREWMANAGMENT Crew;
	int SelectedUmmuMember;				// for the SDK demo, select the member to eva
	int iActionAreaDemoStep;			// this is just to show one feature of action area.
	void clbkSetClassCaps_UMMu(void);	// our special SetClassCap function just added for more readability

	// The HUD display method variable, see PDF doc
	char cUmmuHudDisplay[255];			// UMmu hud char variable
	double dHudMessageDelay;			// UMmu hud display delay
	char *SendHudMessage(void);			// UMmu hud display function

	// "Allow user to add crew to your ship 
	// without scenery editor"
	char cAddUMmuToVessel[255];
	void AddUMmuToVessel(BOOL bStartAdding=FALSE);

	// This section is for UCGO, another terrific development library by Dansteph which allows developers to add cargo carrying capabilities. 

	// UCGO 2.0 CLASS HANDLE FUNCTION AND VARIABLES
	UCGO	hUcgo;						// Cargo class handle
	char   *SendCargHudMessage(void);	// Cargo hud display function
	char	cCargoHudDisplay[255];		// Cargo hud display char variable
	double	dCargHudMessageDelay;		// Cargo hud display delay
	int		iSelectedCargo;				// for the selection of cargos -1 by default

private:
	UINT anim_portNacelle;
	UINT anim_starboardNacelle;

	// Vessel specific parameters are called here like the # of kilos of LOX in the onboard tanks, the positions of the gear & payload bay doors,
	//	a variable that Im hoping to use as a randomizer for this project in the future. Variables are used to store & keep track of various pieces
	// of information during a simulation session, but need to be saved & loaded properly in clbkLoadStateEx & clbkSaveState
	// if they are to be persistent. Oh hi Face... ;)

	double p_rot_angle;
	double s_rot_angle;
	double Randomizer;
};


HINSTANCE hDLL;
HFONT hFont;
HPEN hPen;
HBRUSH hBrush; 
