//=========================================================
// ShuttleD 1.0 source code
// This is what I have been working on for the last few months. The Shuttle-D is currently designed as a NTR propelled cargo transport
// & utility vessel compatible with UMMU, UCGO, & Orbitersound. Ive made sure to heavily comment my source in order to give new addon devs
// a good resource for understanding many of the pitfalls I had to endure, and how to beat them. If youre planning on reading this as a tutorial,
// I would reccomend you start over in D9base.h, as I structured the tutorial starting from there.
//=========================================================

#define STRICT
#define ORBITER_MODULE
#include "1701.h"
#include "UCGOCargoSDK.h"
#include "orbitersdk.h"
#include <math.h>
#include <stdio.h>
#include "OrbiterSoundSDK40.h"
#include "VesselAPI.h"

// these are definitions for sound so functions will be a bit clearer to read..		
#define GEARUP						 1
#define GEARDOWN					 2
#define PLBAYAOPEN					 3
#define PLBAYACLOSE					 4
#define PLBAYBOPEN					 5
#define PLBAYBCLOSE					 6

HINSTANCE g_hDLL;
VISHANDLE MainExternalMeshVisual = 0;

// ==============================================================
// ShuttleD::ShuttleD (OBJHANDLE hObj, int fmodel)
// To the best of my knowledge, this is called when a Shuttle-D is first created & allows parameters to be set to specific values right away.
// For example 	O2Tank = 1000; indicates that the main oxygen tank parameter is set to 1000/1000; full, upon creation of a new Shuttle-D.
// ==============================================================

Enterprise::Enterprise (OBJHANDLE hObj, int fmodel)
	: VESSEL3 (hObj, fmodel)
{
	DefineAnimations();
}

//=========================================================
// Vessel Animations
// This part of the code refers to the nuts & bolts involved in givig this craft animated parts, in this case retractable/extendable gear,
// as well as two payload bay doors which can be opened or closed. 
//=========================================================
void Enterprise::DefineAnimations()
{
	static UINT groupp[1] = {39};
	static MGROUP_ROTATE pnac
	(
		0,
		groupp,1,
		_V(-38.317001,20.182944,-5.519),
		_V(0,0,-1),
		(float)(2.0*PI)
	);
	anim_portNacelle = CreateAnimation(0.0);
	AddAnimationComponent(anim_portNacelle,0,1,&pnac);
	
	static UINT groups[1] = {38};
	static MGROUP_ROTATE snac
	(
		0,
		groups,1,
		_V(38.900002,20.218944,-5.5689),
		_V(0,0,1),
		(float)(2.0*PI)
	);
	anim_starboardNacelle = CreateAnimation(0.0);
	AddAnimationComponent(anim_starboardNacelle,0,1,&snac);
}

//=========================================================
// Mass Update Function
// A big prize to anyone who can guess what this does! Yes that is correct, it updates the mass of a Shuttle-D vessel based on its empty mass & the amount of O2 it is carrying!
// As a previous English teacher I once had was fond of saying, your prize is knoledge & undertanding (shows how much I learned :)
// This function does not fully complete the mass calculations, as the UCGO mass update function down in void clbkPostStep also has a hand in it, but this is the main place or things of that ilk, so...
//=========================================================

double Enterprise::UpdateMass ()
{
double mass=(EXP_EMPTYMASS);
double CrewComplement = Crew.GetCrewTotalNumber();
mass+=(CrewComplement*85);

return mass;
}

//=========================================================
// Vessel Capabilities
// This is what you might consider the core of the addon. In clbkSetClassCaps, all of the core information about the way a vessel behaves
// as a physics object is declared. This includes empty mass, PMI (a measure of the dynamics involved in rotating the vessel), cross-sections,
// Albedo (the colour of the white dot when you zoom out), gear friction coefficients, thrusters & engines, the size of the vessel,
// & of course its mesh (what it looks like). Also included here in this addon are the UMMU & UCGO calls which specify how the vessel uses
// those libraries. They are fairly self-explanatory, so I wont go into detail on them here.
//=========================================================
void Enterprise::clbkSetClassCaps (FILEHANDLE cfg)
{
	Crew.InitUmmu(GetHandle());	
	Crew.DefineAirLockShape(TRUE,-2,2,22,24,60,64);
	Crew.SetMembersPosRotOnEVA(_V(0,23,62),_V(0,-225,0));
	float UMmuVersion=Crew.GetUserUMmuVersion();
	Crew.SetMaxSeatAvailableInShip(40);
	
	//Dock0 = CreateDock(_V(0,23,60),_V(0,0,-1),_V(0,1,0));

	COLOUR4 col_a = {0,0,0,0};
	COLOUR4 col_white = {1,1,1,0};
	double umbra    = RAD*25;
	double penumbra = RAD*90;
	AddSpotLight(_V(0,22,79), _V(0,0,1), 300, 1e-3, 0, 1e-3, umbra, penumbra, col_white, col_white, col_a);
	AddSpotLight(_V(-12,22,70), _V(-1,0,0), 300, 1e-3, 0, 1e-3, umbra, penumbra, col_white, col_white, col_a);
	AddSpotLight(_V(12,22,70), _V(1,0,0), 300, 1e-3, 0, 1e-3, umbra, penumbra, col_white, col_white, col_a);
	AddSpotLight(_V(0,17.8,45.6), _V(0,0,-1), 300, 1e-3, 0, 1e-3, umbra, penumbra, col_white, col_white, col_a);

	SelectedUmmuMember =0;  // our current selected member


	// The HUD display method variables, see PDF doc
	cUmmuHudDisplay[0] =0;	// Initialisation of UMmu hud char variable
	dHudMessageDelay =0;	// Initialisation of UMmu delay variable
	strcpy(SendHudMessage(),"Welcome aboard. Press E to EVA, 1/2 to select crew, 0 or 8 for info, and M to add crew.");

	// The Add mmu without scenery editor variable see PDF doc
	cAddUMmuToVessel[0]=0;

	//UCGO 2.0 Initialisation, cargo slot pos, rot declaration
	hUcgo.Init(GetHandle());
	// in contrary of the PDF code we declare 6 slots
	// because it's fun :)
	hUcgo.DeclareCargoSlot(0,_V(-0.65,-3.3,11.53),_V(0,0,270)); // slot 0
	hUcgo.DeclareCargoSlot(1,_V(-0.65,-3.3,10.23),_V(0,0,270)); // slot 1
	hUcgo.DeclareCargoSlot(2,_V(-0.65,-3.3,8.93),_V(0,0,270)); // slot 2
	hUcgo.DeclareCargoSlot(3,_V(-0.65,-3.3,7.63),_V(0,0,270)); // slot 3
	hUcgo.DeclareCargoSlot(4,_V(-0.65,-3.3,6.33),_V(0,0,270)); // slot 4
	hUcgo.DeclareCargoSlot(5,_V(-0.65,-3.3,5.03),_V(0,0,270)); // slot 5
	hUcgo.DeclareCargoSlot(6,_V(-0.65,-3.3,3.73),_V(0,0,270)); // slot 6
	hUcgo.DeclareCargoSlot(7,_V(-0.65,-3.3,2.43),_V(0,0,270)); // slot 7
	hUcgo.DeclareCargoSlot(8,_V(-0.65,-3.3,1.13),_V(0,0,270)); // slot 8
	hUcgo.DeclareCargoSlot(9,_V(-0.65,-3.3,-0.17),_V(0,0,270)); // slot 9
	hUcgo.DeclareCargoSlot(10,_V(-0.65,-3.3,-1.47),_V(0,0,270)); // slot 10
	hUcgo.DeclareCargoSlot(11,_V(-0.65,-3.3,-2.77),_V(0,0,270)); // slot 11
	hUcgo.DeclareCargoSlot(12,_V(-0.65,-3.3,-4.07),_V(0,0,270)); // slot 12
	hUcgo.DeclareCargoSlot(13,_V(-0.65,-3.3,-5.37),_V(0,0,270)); // slot 13
	hUcgo.DeclareCargoSlot(14,_V(-0.65,-3.3,-6.67),_V(0,0,270)); // slot 14
	hUcgo.DeclareCargoSlot(15,_V(-0.65,-3.3,-7.97),_V(0,0,270)); // slot 15
	hUcgo.DeclareCargoSlot(16,_V(-0.65,-3.3,-9.27),_V(0,0,270)); // slot 16
	hUcgo.DeclareCargoSlot(17,_V(-0.65,-3.3,-10.57),_V(0,0,270)); // slot 17

	hUcgo.SetSlotGroundReleasePos(0,_V(2,-3.6,11.53)); // slot 0
	hUcgo.SetSlotGroundReleasePos(1,_V(2,-3.6,10.23)); // slot 1
	hUcgo.SetSlotGroundReleasePos(2,_V(2,-3.6,8.93)); // slot 2
	hUcgo.SetSlotGroundReleasePos(3,_V(2,-3.6,7.63)); // slot 3
	hUcgo.SetSlotGroundReleasePos(4,_V(2,-3.6,6.33)); // slot 4
	hUcgo.SetSlotGroundReleasePos(5,_V(2,-3.6,5.03)); // slot 5
	hUcgo.SetSlotGroundReleasePos(6,_V(2,-3.6,3.73)); // slot 6
	hUcgo.SetSlotGroundReleasePos(7,_V(2,-3.6,2.43)); // slot 7
	hUcgo.SetSlotGroundReleasePos(8,_V(2,-3.6,1.13)); // slot 8
	hUcgo.SetSlotGroundReleasePos(9,_V(2,-3.6,-0.17)); // slot 9
	hUcgo.SetSlotGroundReleasePos(10,_V(2,-3.6,-1.47)); // slot 10
	hUcgo.SetSlotGroundReleasePos(11,_V(2,-3.6,-2.77)); // slot 11
	hUcgo.SetSlotGroundReleasePos(12,_V(2,-3.6,-4.07)); // slot 12
	hUcgo.SetSlotGroundReleasePos(13,_V(2,-3.6,-5.37)); // slot 13
	hUcgo.SetSlotGroundReleasePos(14,_V(2,-3.6,-6.67)); // slot 14
	hUcgo.SetSlotGroundReleasePos(15,_V(2,-3.6,-7.97)); // slot 15
	hUcgo.SetSlotGroundReleasePos(16,_V(2,-3.6,-9.27)); // slot 16
	hUcgo.SetSlotGroundReleasePos(17,_V(2,-3.6,-10.57)); // slot 17

	// UCGO 2.0 Parameters settings
	hUcgo.SetReleaseSpeedInSpace(0.001f);	   // release speed of cargo in space in m/s
	hUcgo.SetMaxCargoMassAcceptable(50000.0);   // max cargo mass in kg that your vessel can carry
	hUcgo.SetGrappleDistance(60);		       // grapple distance radius in meter from center of ship


	// UCGO Variables initialisation
	cCargoHudDisplay[0]=0;						// Cargo hud display char variable
	dCargHudMessageDelay=0;						// Cargo hud display delay
	iSelectedCargo=-1;							// for the selection of cargos (-1 mean "default" see header)
	// welcome message with keys for users
	strcpy(SendCargHudMessage(),"Payload Controls C/Shift+C to grapple/release, 9/Shift+9 to add cargo.");

	DOCKHANDLE Dock0;

	// vessel caps definitions

	SetMeshVisibilityMode (AddMesh (DVCInterior = oapiLoadMeshGlobal ("TOSEnterprise")), MESHVIS_ALWAYS);
	

	SetCameraOffset (_V(0,25.245,76.269));
	SetAlbedoRGB (_V(0.77,0.20,0.73));
	SetSize (EXP_SIZE);



	SetPMI (EXP_PMI);
	SetCrossSections (EXP_CS);
	SetSurfaceFrictionCoeff (0.55, 0.79);
	SetRotDrag (_V(0.15,0.2,0.15));
	SetTouchdownPoints(_V(10,-40,10),_V(-10,-40,10),_V(0,-40,-30));

	EnableTransponder (true);
	InitNavRadios (4);


	// propellant resources
	PROPELLANT_HANDLE dilith = CreatePropellantResource(12.2e6);
	PROPELLANT_HANDLE reac   = CreatePropellantResource(1000);

	THRUSTER_HANDLE impulse, gravity, warp, th_rcs[14], th_group[4];

	SURFHANDLE redex = oapiRegisterExhaustTexture("Exhaustred");
	SURFHANDLE blueex= oapiRegisterExhaustTexture("Exhaustblue");

	impulse = CreateThruster(_V(0,0,0),_V(0,0,1),6e9,dilith,9e9);
	CreateThrusterGroup(&impulse,1,THGROUP_MAIN);
	AddExhaust(impulse,60,6,_V(-4.1,10.5,3.6),_V(0,0,-1),redex);
	AddExhaust(impulse,60,6,_V(4.1,10.5,3.6),_V(0,0,-1),redex);

	gravity = CreateThruster(_V(0,0,0),_V(0,1,0),1e10,dilith,800e12);
	CreateThrusterGroup(&gravity,1,THGROUP_HOVER);

	warp = CreateThruster(_V(0,0,0),_V(0,0,1),9e12,dilith,9e9);
	CreateThrusterGroup(&warp,1,THGROUP_RETRO);
	AddExhaust(warp,120,10,_V(-38.75,20.20,-147.09),_V(0,0,-1),blueex);
	AddExhaust(warp,120,10,_V( 38.75,20.20,-147.09),_V(0,0,-1),blueex);

	th_rcs[ 0] = CreateThruster (_V( 100,0, 300), _V(0, 1,0), 6e8, reac, 9e9); //front right bottom
	th_rcs[ 1] = CreateThruster (_V( 100,0, 300), _V(0,-1,0), 6e8, reac, 9e9); //front right top
	th_rcs[ 2] = CreateThruster (_V(-100,0, 300), _V(0, 1,0), 6e8, reac, 9e9); //front left bottom
	th_rcs[ 3] = CreateThruster (_V(-100,0, 300), _V(0,-1,0), 6e8, reac, 9e9); //front left top
	th_rcs[ 4] = CreateThruster (_V( 100,0,-300), _V(0, 1,0), 6e8, reac, 9e9); //back right bottom
	th_rcs[ 5] = CreateThruster (_V( 100,0,-300), _V(0,-1,0), 6e8, reac, 9e9); //back right top
	th_rcs[ 6] = CreateThruster (_V(-100,0,-300), _V(0, 1,0), 6e8, reac, 9e9); //back left bottom
	th_rcs[ 7] = CreateThruster (_V(-100,0,-300), _V(0,-1,0), 6e8, reac, 9e9); //back left top
	th_rcs[ 8] = CreateThruster (_V( 100,0, 300), _V(-1,0,0), 6e8, reac, 9e9); //front right right
	th_rcs[ 9] = CreateThruster (_V(-100,0, 300), _V( 1,0,0), 6e8, reac, 9e9); //argh too many thrusters
	th_rcs[10] = CreateThruster (_V( 100,0,-300), _V(-1,0,0), 6e8, reac, 9e9);
	th_rcs[11] = CreateThruster (_V(-100,0,-300), _V( 1,0,0), 6e8, reac, 9e9);
	th_rcs[12] = CreateThruster (_V( 0,0,-300), _V(0,0, 1),  6e8, reac, 9e9);
	th_rcs[13] = CreateThruster (_V( 0,0, 300), _V(0,0,-1),  6e8, reac, 9e9);

	th_group[0] = th_rcs[0];
	th_group[1] = th_rcs[2];
	th_group[2] = th_rcs[5];
	th_group[3] = th_rcs[7];
	CreateThrusterGroup (th_group, 4, THGROUP_ATT_PITCHUP);

	th_group[0] = th_rcs[1];
	th_group[1] = th_rcs[3];
	th_group[2] = th_rcs[4];
	th_group[3] = th_rcs[6];
	CreateThrusterGroup (th_group, 4, THGROUP_ATT_PITCHDOWN);

	th_group[0] = th_rcs[0];
	th_group[1] = th_rcs[4];
	th_group[2] = th_rcs[3];
	th_group[3] = th_rcs[7];
	CreateThrusterGroup (th_group, 4, THGROUP_ATT_BANKLEFT);

	th_group[0] = th_rcs[1];
	th_group[1] = th_rcs[5];
	th_group[2] = th_rcs[2];
	th_group[3] = th_rcs[6];
	CreateThrusterGroup (th_group, 4, THGROUP_ATT_BANKRIGHT);

	th_group[0] = th_rcs[0];
	th_group[1] = th_rcs[4];
	th_group[2] = th_rcs[2];
	th_group[3] = th_rcs[6];
	CreateThrusterGroup (th_group, 4, THGROUP_ATT_UP);

	th_group[0] = th_rcs[1];
	th_group[1] = th_rcs[5];
	th_group[2] = th_rcs[3];
	th_group[3] = th_rcs[7];
	CreateThrusterGroup (th_group, 4, THGROUP_ATT_DOWN);

	th_group[0] = th_rcs[8];
	th_group[1] = th_rcs[11];
	CreateThrusterGroup (th_group, 2, THGROUP_ATT_YAWLEFT);

	th_group[0] = th_rcs[9];
	th_group[1] = th_rcs[10];
	CreateThrusterGroup (th_group, 2, THGROUP_ATT_YAWRIGHT);

	th_group[0] = th_rcs[8];
	th_group[1] = th_rcs[10];
	CreateThrusterGroup (th_group, 2, THGROUP_ATT_LEFT);

	th_group[0] = th_rcs[9];
	th_group[1] = th_rcs[11];
	CreateThrusterGroup (th_group, 2, THGROUP_ATT_RIGHT);

	CreateThrusterGroup (th_rcs+12, 1, THGROUP_ATT_FORWARD);
	CreateThrusterGroup (th_rcs+13, 1, THGROUP_ATT_BACK);

	Dock0 = CreateDock(_V(0,23,60),_V(0,0,-1),_V(0,1,0));

}
//=========================================================
// clbkDrawHUD
// This code helps in drawing the custom hud displays that UMMU & UCGO use to send messages. All I really know about this at this point is that in
//   5,hps->H/60*25,cUmmuHudDisplay 
// 60 & 25 appear to change where the text appears on the display.
//=========================================================

bool Enterprise::clbkDrawHUD(int mode, const HUDPAINTSPEC *hps, oapi::Sketchpad *skp)
{
	// draw the default HUD
	VESSEL3::clbkDrawHUD (mode, hps, skp);

	// UMmu display messages
	if(dHudMessageDelay>0)
	{
		skp->Text(5,hps->H/60*25,cUmmuHudDisplay,strlen(cUmmuHudDisplay));
		dHudMessageDelay-=oapiGetSimStep();
		if(dHudMessageDelay<0)
			dHudMessageDelay=0;
	}
	// UCGO display messages
	if(dCargHudMessageDelay>0)
	{
		skp->Text(5,hps->H/60*22.5,cCargoHudDisplay,strlen(cCargoHudDisplay));
		dCargHudMessageDelay-=oapiGetSimStep();
		if(dCargHudMessageDelay<0)
			dCargHudMessageDelay=0;
	}
	return true; 
}
char *Enterprise::SendHudMessage() //<---- Change the class name here
{
	dHudMessageDelay=15;
	return cUmmuHudDisplay;
}

//=========================================================
// clbkLoadStateEx
// I would like to dedicate this as the Face function, lol. A big thanks to Face who helped me fix this part of the code when it wasnt written properly,
// and was causing me a lot of grief. The key thing to remember here and in clbkSaveState is that when custom variables (data about your ship) gets saved
// it gets a line in the SCN file. In say,
//
//		if (!_strnicmp (line, "O2Tank", 6)) {
//			sscanf (line+6, "%lf", &O2Tank);
//		}
//
// The number 6 refers to the number of characters in O2Tank - 6. This is the length of the id string, NOT how many lines it has to slip down in the
// scn file.
//=========================================================
void Enterprise::clbkLoadStateEx (FILEHANDLE scn, void *status)
{
	char *line;
	while (oapiReadScenario_nextline (scn, line)) 
	{

		if(Crew.LoadAllMembersFromOrbiterScenario(line)==TRUE)
			continue;

		// Load UCGO 2.0 cargo from scenario
		if(hUcgo.LoadCargoFromScenario(line)==TRUE) // UCGO load cargo 
			continue;

		ParseScenarioLineEx (line, status);
	}

}

//=========================================================
// clbkSaveState
// Nothing too exciting here, just the same rules as above, but it is worth noting that in "%d %0.4f", the d & f appear to specify
// the data type being saved. In other words, the specifier "%d" or "%0.4f" will determine how many zeroes get written to the SCN, &
// as a result remembered.
//=========================================================

void Enterprise::clbkSaveState (FILEHANDLE scn)
{
	char cbuf[256];

	VESSEL3::clbkSaveState (scn);

	Crew.SaveAllMembersInOrbiterScenarios(scn);

	// Save UCGO 2.0 cargo in scenario
	hUcgo.SaveCargoToScenario(scn);	
}

//=========================================================
// clbkVisualCreated
// Clear as mud to me, but I think UCGO uses it when adding the attached cargo meshes to the simulation. Dansteph probably knows...
//=========================================================

void Enterprise::clbkVisualCreated (VISHANDLE vis, int refcount)
{
	hUcgo.SetUcgoVisual(vis);	// must be called in clbkVisualCreated.
}

//=========================================================
// SendCargHudMessage
// Another UCGO specific function, which you would think would be related to the hud.
//=========================================================

char *Enterprise::SendCargHudMessage(void)
{
	dCargHudMessageDelay=15; // 15 seconds display delay for msg
	return cCargoHudDisplay;
}

//=========================================================
// clbkPostStep
// This is the step-to-step, always-in-motion part of the code during an Orbiter simulation. Orbiter does physics caculations every timestep,
// and as a result, interesting things can be done here like constant updating of variables, the nuts & bolts that drive the animations up & down,
// as well as functions to kill the crew when crashing or running out of air. I wont go into great detail on what I have in here, but this tends to be
// the exciting part of the code. This is where things happen!
//=========================================================

void Enterprise::clbkPostStep(double simt, double simdt, double mjd)
{

SetEmptyMass(UpdateMass());
hUcgo.UpdateEmptyMass();

	int ReturnCode=Crew.ProcessUniversalMMu();
	switch(ReturnCode)
	{
	case UMMU_TRANSFERED_TO_OUR_SHIP: 
		sprintf(SendHudMessage(),"%s \"%s\" transfered to %s",
			Crew.GetCrewMiscIdByName(Crew.GetLastEnteredCrewName()),Crew.GetLastEnteredCrewName()
			,GetName());
		break;
	case UMMU_RETURNED_TO_OUR_SHIP:
		sprintf(SendHudMessage(),"%s \"%s\" ingressed %s",
			Crew.GetCrewMiscIdByName(Crew.GetLastEnteredCrewName()),
			Crew.GetLastEnteredCrewName(),GetName());
		break;
	}

		//Fiddly animation stuff
	double s = simdt * rot_speed;
	p_rot_angle += s;
	if (p_rot_angle > 1.0)
		p_rot_angle -= floor(p_rot_angle);
	SetAnimation(anim_portNacelle,p_rot_angle);

	s_rot_angle += s;
	if (s_rot_angle > 1.0)
		s_rot_angle -= floor(s_rot_angle);
	SetAnimation(anim_starboardNacelle,s_rot_angle);


	if(GroundContact()==TRUE)
	{
		// we check vertical speed
		int I;
		VECTOR3 vHorizonAirspeedVector={0};
		GetHorizonAirspeedVector (vHorizonAirspeedVector);
		double VertSpeed		=vHorizonAirspeedVector.y;
		if(VertSpeed<-3)
		{
			// we touched ground with more than -3 m/s, sorry dude, time to kill you all :(
			for(I=0;I<Crew.GetCrewTotalNumber();I++)
			{
				Crew.SetCrewMemberPulseBySlotNumber(I,0);	// set cardiac pulse to zero
			}
			strcpy(SendHudMessage(),"Crashed into terrain");
		}
	}

		// Detect Atmospheric Entry-Function Courtesy of Hlynkacg
	if ( (GetDynPressure() > 120000) ) // It's about to get very hot in here...
	{
			int I;
			// This procedure wasnt in the owners manual... What do you mean we dont have a heat shield?!?!?
			for(I=0;I<Crew.GetCrewTotalNumber();I++)
			{
				Crew.SetCrewMemberPulseBySlotNumber(I,0);	// set cardiac pulse to zero
			}
			strcpy(SendHudMessage(),"Hull Breach due to Atmospheric Reentry");
	}

	hUcgo.SetCargoMeshVisibility(17,MESHVIS_NEVER);

	Randomizer = sin(simdt);

		AddUMmuToVessel();

		Crew.WarnUserUMMUNotInstalled("Enterprise");
		// At the very end of clbkPostStep or clbkPreStep
		hUcgo.WarnUserUCGONotInstalled("Enterprise");
}


//=========================================================
// clbkConsumeBufferedKey
// Another important function, this particular section is used to execute functions whenever the Orbiter application detects a given keystroke by a user.
// Really not very complicated (usually), very similar to clbkVCMouseEvent. The ony complicated part is getting the structure right, as calls specific
// to a ctrl-keypress or shift-keypress have to be grouped under a separate heading. Best example of that will be under the Hubble Space Telescope sample
// also in the OrbiterSDK directory.
//=========================================================

int Enterprise::clbkConsumeBufferedKey (DWORD key, bool down, char *kstate)
{

	if (!down) return 0;       // only process keydown events 

	if(key==OAPI_KEY_E&&!KEYMOD_SHIFT(kstate)&&!KEYMOD_CONTROL (kstate))
	{
		// PERFORM THE EVA, first we get is name with "GetCrewNameBySlotNumber" then we perform EVA with "EvaCrewMember"
		int Returned=Crew.EvaCrewMember(Crew.GetCrewNameBySlotNumber(SelectedUmmuMember));
		// we provide feedback to user (You can display a message on panel or wathewer)
		// here below all the return code possible:
		switch(Returned)
		{
		case TRANSFER_TO_DOCKED_SHIP_OK:
			sprintf(SendHudMessage(),"%s transfered through main hatch",
				Crew.GetLastEvaedCrewName());SelectedUmmuMember=0;
			break;
		case EVA_OK:
			sprintf(SendHudMessage(),"%s on EVA",
				Crew.GetLastEvaedCrewName());SelectedUmmuMember=0;
			break;
		case ERROR_AIRLOCK_CLOSED:
			strcpy(SendHudMessage(),"Airlock closed. press A to open");
			break;
		case ERROR_DOCKED_SHIP_HAVE_AIRLOCK_CLOSED:
			strcpy(SendHudMessage(),"Docked vessel airlock closed");
			break;
		case ERROR_CREW_MEMBER_NOT_FOUND:
			strcpy(SendHudMessage(),"No crew by this name aboard");
			break;
		case ERROR_DOCKEDSHIP_DONOT_USE_UMMU:
			strcpy(SendHudMessage(),"Docked ship is not compatible with UMmu 2.0");
			break;
		case ERROR_MISC_ERROR_EVAFAILED:
			strcpy(SendHudMessage(),"Misc error with UMMU. Please reinstall");
			break;
		}
		return TRUE;
	}

	//---------------------------------------------------------------------------
	// Ummu Key "1" - Select next member This is just internal to the demo
	// you may do your own selection system by panel button, name etc etc
	if(key==OAPI_KEY_1&&!KEYMOD_SHIFT(kstate)&&!KEYMOD_CONTROL (kstate))
	{
		// we test there is someone aboard
		if(Crew.GetCrewTotalNumber()==0)
		{
			strcpy(SendHudMessage(),"No crew aboard");	
			return 1;
		}

		// we test that we select existing member
		if(SelectedUmmuMember<Crew.GetCrewTotalNumber()-1)
			SelectedUmmuMember++;
		char * Name=Crew.GetCrewNameBySlotNumber(SelectedUmmuMember);
		sprintf(SendHudMessage(),"%i  %s \"%s\"Selected for EVA or Transfer",
			SelectedUmmuMember,Crew.GetCrewMiscIdBySlotNumber(SelectedUmmuMember),
			Name);
		return 1;
	}

	//---------------------------------------------------------------------------
	// Ummu Key "2" - Select previous member This is just internal to the demo
	// you may do your own selection system by panel button
	if(key==OAPI_KEY_2&&!KEYMOD_SHIFT(kstate)&&!KEYMOD_CONTROL (kstate))
	{
		// we test there is someone aboard
		if(Crew.GetCrewTotalNumber()==0)
		{
			strcpy(SendHudMessage(),"No crew aboard");	
			return 1;
		}
		if(SelectedUmmuMember>0)
			SelectedUmmuMember--;
		char * Name=Crew.GetCrewNameBySlotNumber(SelectedUmmuMember);
		sprintf(SendHudMessage(),"Slot %i %s \"%s\" Selected for EVA or Transfer"
			,SelectedUmmuMember,Crew.GetCrewMiscIdBySlotNumber(SelectedUmmuMember),Name);
		return 1;
	}


		//---------------------------------------------------------------------------
	// UCGO Key "4" Select next Cargo Slot 
	if(key==OAPI_KEY_4&&!KEYMOD_SHIFT(kstate)&&!KEYMOD_CONTROL (kstate))
	{


		// we test that we select existing member
		if(iSelectedCargo<18-1)
			iSelectedCargo++;
		double GetCargoSlotMass(int Num); 
		sprintf(SendCargHudMessage(),"Cargo Slot %i selected",
			iSelectedCargo);
		return 1;
	}

	//---------------------------------------------------------------------------
	// UCGO Key "3" Select previous Cargo Slot
	if(key==OAPI_KEY_3&&!KEYMOD_SHIFT(kstate)&&!KEYMOD_CONTROL (kstate))
	{

		if(iSelectedCargo>0)
			iSelectedCargo--;
		double GetCargoSlotMass(int Num); 
		sprintf(SendCargHudMessage(),"Cargo Slot %i selected",
			iSelectedCargo);
		return 1;
	}

	//---------------------------------------------------------------------------
	// Get some infos Name of ship and total soul aboard
	if(key==OAPI_KEY_0)
	{
		sprintf(SendHudMessage(),"%i crew aboard USS %s",
			Crew.GetCrewTotalNumber(),GetName());
		return 1;
	}

	// THIS IS FOR ADDING CREW SEE PDF doc "Allow user to add crew to your ship 
	// without scenery editor"
	if(key==OAPI_KEY_M&&!KEYMOD_SHIFT(kstate)&&!KEYMOD_CONTROL (kstate))
	{
		AddUMmuToVessel(TRUE);
	}
	// 9 key "select" one cargo on disk (cycle)
	if(key==OAPI_KEY_9&&!KEYMOD_SHIFT(kstate)&&!KEYMOD_CONTROL (kstate))
	{
		sprintf(SendCargHudMessage(),"%s",hUcgo.ScnEditor_SelectNextCargoAvailableOnDisk());
		return 1;
	}
	// SHIFT+9 key "add last cargo selected by key 9"
	// If iSelectedCargo=-1 (default) add to the first free slot found
	if(key==OAPI_KEY_9&&KEYMOD_SHIFT(kstate)&&!KEYMOD_CONTROL (kstate))
	{
		if(hUcgo.ScnEditor_AddLastSelectedCargoToSlot(iSelectedCargo)==TRUE)
		{
			strcpy(SendCargHudMessage(),"Cargo loaded");
		}
		else
		{
			if(iSelectedCargo<0)
			{
				strcpy(SendCargHudMessage(),"Unable to load Cargo");
			}
			else
			{
				strcpy(SendCargHudMessage(),"Unable to load Cargo");
			}
		}
		return 1;
	}
	// "C" grapple cargo. If iSelectedCargo=-1 (default) add to the first free slot found
	if(key==OAPI_KEY_C&&!KEYMOD_SHIFT(kstate)&&!KEYMOD_CONTROL (kstate))
	{
		int ReturnedCode=hUcgo.GrappleOneCargo(iSelectedCargo);
		// for return code list see function "GrappleOneCargo" in the header
		switch(ReturnedCode)
		{
		case 1:
			strcpy(SendCargHudMessage(),"Cargo grappled");
			break;
		case 0:
			strcpy(SendCargHudMessage(),"No cargo in range");
			break;
		case -1:
			strcpy(SendCargHudMessage(),"Maximum payload mass exceeded");
			break;
		case -2:
			strcpy(SendCargHudMessage(),"bad config, mesh not found or slot not declared");
			break;
		case -3:
			strcpy(SendCargHudMessage(),"Cargo slot full, unable to grapple cargo");
			break;
		case -4:
			strcpy(SendCargHudMessage(),"Payload Bay A closed");
			break;
		case -5:
			strcpy(SendCargHudMessage(),"Unable to load Cargo. All slots full");
			break;
		default:
			strcpy(SendCargHudMessage(),"Misc error. Unable to grapple cargo");
		}
		return 1;
	}
	// SHIFT+C release cargo. If iSelectedCargo=-1 (default) release the first free slot found
	if(key==OAPI_KEY_C&&KEYMOD_SHIFT(kstate)&&!KEYMOD_CONTROL (kstate))
	{
		if(hUcgo.ReleaseOneCargo(iSelectedCargo)!=FALSE)
		{
			strcpy(SendCargHudMessage(),"Cargo release");
		}
		else
		{
			strcpy(SendCargHudMessage(),"Slot empty");
		}
		return 1;
	}
	// "8" show some info on cargo
	if(key==OAPI_KEY_8&&!KEYMOD_SHIFT(kstate)&&!KEYMOD_CONTROL (kstate))
	{
		sprintf(SendCargHudMessage(),"Curent Payload mass %.0fkg in "
			"%i slots",hUcgo.GetCargoTotalMass(),hUcgo.GetNbrCargoLoaded());
	return 1;
	}

	return 0;
}

//=========================================================
// UMmuCrewAddCallback & AddUMmuToVessel
// Again, a Dansteph creation, so I dont know a great deal about it, but its used in adding crew directly to the ship without entering through the main
// hatch. Fairly usefull, Id say.
//=========================================================

bool UMmuCrewAddCallback(void *id, char *str, void *data)
{
	if(strlen(str)<2||strlen(str)>38)
		return false;
	char *cPtr=(char*)data;	if(*cPtr==2){*cPtr=3;strcpy(cPtr+2,str);}
	else if(*cPtr==4){*cPtr=5;strcpy(cPtr+42,str);}
	else if(*cPtr==6){*cPtr=7;strcpy(cPtr+82,str);}return true;
}
void Enterprise::AddUMmuToVessel(BOOL bStartAdding)
{
	if(bStartAdding==FALSE&&cAddUMmuToVessel[0]==0)
		return;
	if(bStartAdding==TRUE){
		int salut=sizeof(cAddUMmuToVessel);
		memset(cAddUMmuToVessel,0,sizeof(cAddUMmuToVessel));
		cAddUMmuToVessel[0]=1;
	}
	else if(cAddUMmuToVessel[0]==1){
		cAddUMmuToVessel[0]=2;
		oapiOpenInputBox ("Enter new crew member name (or hit escape to cancel)",UMmuCrewAddCallback,0,30,(void*)cAddUMmuToVessel);
	}
	else if(cAddUMmuToVessel[0]==3){
		cAddUMmuToVessel[0]=4;
		oapiOpenInputBox ("Enter age",UMmuCrewAddCallback,0,30,(void*)cAddUMmuToVessel);
	}
	else if(cAddUMmuToVessel[0]==5){
		cAddUMmuToVessel[0]=6;
		oapiOpenInputBox ("Enter Crew ID - Capt,Sec,Vip,Sci,Doc,Tech,Crew,Pax)",UMmuCrewAddCallback,0,30,(void*)cAddUMmuToVessel);
	}
	else if(cAddUMmuToVessel[0]==7){
		cAddUMmuToVessel[0]=0;
		int Age=max(5,min(100,atoi(&cAddUMmuToVessel[42])));
		if(Crew.AddCrewMember(&cAddUMmuToVessel[2],Age,70,70,&cAddUMmuToVessel[82])==TRUE){
			sprintf(SendHudMessage(),"\"%s\" aged %i added to vessel",&cAddUMmuToVessel[2],Age);
		}
		else{
			strcpy(SendHudMessage(),"Unable to add crew");
		}
	}
}



//=========================================================
// Load and Delete Module Stuff
// Apparenty used to load & delete special classes in the module. If theyre not deleted properly in ExitModule, they cause what I believe is called a
// memory leak, not good. So always remember to clean up after yourself!
//=========================================================
DLLCLBK void InitModule (HINSTANCE hModule)
{
	g_hDLL = hModule;
	hFont = CreateFont (-20, 3, 0, 0, 150, 0, 0, 0, 0, 0, 0, 0, 0, "Haettenschweiler");
	hPen = CreatePen (PS_SOLID, 3, RGB (120,220,120));
	hBrush = CreateSolidBrush (RGB(0,128,0));

	// perform global module initialisation here
}
DLLCLBK void ExitModule (HINSTANCE hModule)
{
	// perform module cleanup here
	DeleteObject (hFont);
	DeleteObject (hPen);
	DeleteObject (hBrush);

}

//=========================================================
// Load and Delete Vessel Stuff
// Appears similar to the part above, only I think its involved when creating or deleting vessels via the scenario editor.
//=========================================================
DLLCLBK VESSEL *ovcInit (OBJHANDLE hvessel, int flightmodel)
{
	return new Enterprise (hvessel, flightmodel);
}

DLLCLBK void ovcExit (VESSEL *vessel)
{
	if (vessel)
		delete (Enterprise*)vessel;
}

//=========================================================
// clbkPostCreation
// This is usually Orbitersound territory. Im not terribly familiar with how it works in 3.5 or 4.0, so I wont say much here. Better just to look
// up Dantephs tutoials in the Orbitersound docs. He can explain a lot of things better than me ;)
//=========================================================

void Enterprise::clbkPostCreation (void)
{

	////////////////////////////////////////////////////////////////
	// 3-ORBITERSOUND EXAMPLE - INIT AND LOADING OF WAV
	// THIS MUST BE CALLED ABSOLUTELY IN THE "POSTCREATION CALLBACK" 
	////////////////////////////////////////////////////////////////
	// here we connect to OrbiterSound and store the returned ID in your class
	// this is the first thing to do. You must call this in "clbkPostCreation" 
	// (new version of ovcPostCreation wich is now obsolet)
	ENT=ConnectToOrbiterSoundDLL(GetHandle());

	SetMyDefaultWaveDirectory("Sound\\_CustomVesselsSounds\\Enterprise\\");

	// now we are allowed for example to replace variable sound of OrbiterSound.
	// it will use them instead of the stock one when our vessel have the focus, see header file for parameter 
	// (you can replace more than the four below see parameters for "ReplaceStockSound3()")
	ReplaceStockSound(ENT,"mainext.wav",	REPLACE_MAIN_THRUST);
	ReplaceStockSound(ENT,"attfire.wav",		REPLACE_RCS_THRUST_ATTACK);
	ReplaceStockSound(ENT,"attsustain.wav",	REPLACE_RCS_THRUST_SUSTAIN);
	ReplaceStockSound(ENT,"aircond.wav",		REPLACE_AIR_CONDITIONNING);

	ReplaceStockSound(ENT,"VCamb1.wav",		REPLACE_COCKPIT_AMBIENCE_1);
	ReplaceStockSound(ENT,"VCamb2.wav",		REPLACE_COCKPIT_AMBIENCE_2);
	ReplaceStockSound(ENT,"VCamb3.wav",		REPLACE_COCKPIT_AMBIENCE_3);
	ReplaceStockSound(ENT,"VCamb4.wav",		REPLACE_COCKPIT_AMBIENCE_4);
	ReplaceStockSound(ENT,"VCamb5.wav",		REPLACE_COCKPIT_AMBIENCE_5);
	ReplaceStockSound(ENT,"VCamb6.wav",		REPLACE_COCKPIT_AMBIENCE_6);
	ReplaceStockSound(ENT,"VCamb7.wav",		REPLACE_COCKPIT_AMBIENCE_7);
	ReplaceStockSound(ENT,"aircond.wav",		REPLACE_COCKPIT_AMBIENCE_8);
	ReplaceStockSound(ENT,"aircond.wav",		REPLACE_COCKPIT_AMBIENCE_9);

	SoundOptionOnOff(ENT,PLAYRADIOATC,FALSE);
}

//=========================================================
// ~ShuttleD
// This is what I believe is called the destructor. A better name for it might be the Terminator, because Ill be back!!!
//=========================================================

Enterprise::~Enterprise() 
{
}

//=========================================================
// Hope the work I did commenting this source helped, please let me know what you think via email at johnnybgoode@rogers.com or on the Orbiter Forums
// at BruceJohnJennerLawso (or even my Shuttle-D development thread). This project has been about 10 months or so in the making & I
// hope you enjoy flying the Shuttle-D, as much as I enjoyed creating it. Stay tuned for 1.1, and
// 
// Hail the Probe!
//
//=========================================================