//*****************************************************************
// UCGO 2.0 SDK EXAMPLE by Dansteph
// PLEASE SEE PDF DOC
// ALL SPECIFIC UCGO CODE ARE IN THIS CPP
// AND IN CLASS DECLARATION IN ShuttlePB.h
//*****************************************************************
#include "D9base.h"


// --------------------------------------------------------------
// Set vessel class caps 
// this is a custom clbkSetClassCaps
// called form the real one in ShuttlePB.cpp this to keep
// only UCGO specific code in this one.
// --------------------------------------------------------------
void ShuttleD::clbkSetClassCapsUCgo (void)
{
	//UCGO 2.0 Initialisation, cargo slot pos, rot declaration
	hUcgo.Init(GetHandle());
	// in contrary of the PDF code we declare 6 slots
	// because it's fun :)
	hUcgo.DeclareCargoSlot(0,_V(0,2.6,-3.50),_V(0,0,0)); // slot 0
	hUcgo.DeclareCargoSlot(1,_V(0,1.3,-3.50),_V(0,0,90)); // slot 1
	hUcgo.DeclareCargoSlot(2,_V(0,0,-3.50),_V(0,0,180)); // slot 2 
	hUcgo.DeclareCargoSlot(3,_V( 0,-1.3,-3.50),_V(0,0,270)); // slot 3 
	hUcgo.DeclareCargoSlot(4,_V(0,-2.6,-3.50),_V(0,0,0)); // slot 4 
	hUcgo.DeclareCargoSlot(5,_V( 0,-3.9,-3.50),_V(0,0,0)); // slot 5 

	// UCGO 2.0 Parameters settings
	hUcgo.SetReleaseSpeedInSpace(0.10f);	   // release speed of cargo in space in m/s
	hUcgo.SetMaxCargoMassAcceptable(10000.0);   // max cargo mass in kg that your vessel can carry
	hUcgo.SetGrappleDistance(50);		       // grapple distance radius in meter from center of ship
	hUcgo.SetSlotGroundReleasePos(0 ,_V(-4,2.6,-3.50));
	hUcgo.SetSlotGroundReleasePos(1 ,_V(-4,1.3,-3.50));
	hUcgo.SetSlotGroundReleasePos(2 ,_V(-4,0,-3.50));
	hUcgo.SetSlotGroundReleasePos(3 ,_V(-4,-1.3,-3.50));
	hUcgo.SetSlotGroundReleasePos(4 ,_V(-4,-2.6,-3.50));
	hUcgo.SetSlotGroundReleasePos(5 ,_V(-4,-3.9,-3.50));

	// UCGO Variables initialisation
	cCargoHudDisplay[0]=0;						// Cargo hud display char variable
	dCargHudMessageDelay=0;						// Cargo hud display delay
	iSelectedCargo=-1;							// for the selection of cargos (-1 mean "default" see header)
	// welcome message with keys for users
	strcpy(SendCargHudMessage(),"Cargo key: C/SHF+C = grapple/release, 9/SHF+9 = snc editor add cargo, 8=infos on cargos");
}
// --------------------------------------------------------------
// Write status to scenario file
// --------------------------------------------------------------
void ShuttleD::clbkSaveState (FILEHANDLE scn)
{
	// default vessel parameters
	VESSEL3::clbkSaveState (scn);

	// Save UCGO 2.0 cargo in scenario
	hUcgo.SaveCargoToScenario(scn);	
}
// --------------------------------------------------------------
// Read status from scenario file
// --------------------------------------------------------------
void ShuttleD::clbkLoadStateEx (FILEHANDLE scn, void *vs)
{
	char *line;
	while (oapiReadScenario_nextline (scn, line)) 
	{
		// Load UCGO 2.0 cargo from scenario
		if(hUcgo.LoadCargoFromScenario(line)==TRUE) // UCGO load cargo 
			continue;	

		// unrecognised option - pass to Orbiter's generic parser
		ParseScenarioLineEx (line, vs);
	}
}
// --------------------------------------------------------------
// clbkVisualCreated
// --------------------------------------------------------------
void ShuttleD::clbkVisualCreated (VISHANDLE vis, int refcount)
{
	hUcgo.SetUcgoVisual(vis);	// must be called in clbkVisualCreated.
}
////////////////////////////////////////////////////////
// SendCargHudMessage
////////////////////////////////////////////////////////
char *ShuttleD::SendCargHudMessage(void)
{
	dCargHudMessageDelay=15; // 15 seconds display delay for msg
	return cCargoHudDisplay;
}
// --------------------------------------------------------------
// Frame update
// --------------------------------------------------------------
void ShuttleD::clbkPostStep (double simt, double simdt, double mjd)
{	
     // At the very end of clbkPostStep or clbkPreStep
     hUcgo.WarnUserUCGONotInstalled("ShuttlePb Ucgo");
}
// --------------------------------------------------------------
// Setup the virtual cockpit instruments
// --------------------------------------------------------------
void ShuttleD::clbkDrawHUD (int mode, const HUDPAINTSPEC *hps, HDC hDC, oapi::Sketchpad *skp)
{
	// draw the default HUD
	VESSEL2::clbkDrawHUD (mode, hps, hDC);

	// UCGO display messages
	if(dCargHudMessageDelay>0)
	{
		TextOut (hDC,5,hps->H/60*12,cCargoHudDisplay,strlen(cCargoHudDisplay));
		dCargHudMessageDelay-=oapiGetSimStep();
		if(dCargHudMessageDelay<0)
			dCargHudMessageDelay=0;
	}
}
// --------------------------------------------------------------
// Respond to buffered keyboard events
// --------------------------------------------------------------
int ShuttleD::clbkConsumeBufferedKey (DWORD key, bool down, char *kstate)
{
	if (!down) return 0;       // only process keydown events

	// 9 key "select" one cargo on disk (cycle)
	if(key==OAPI_KEY_9&&!KEYMOD_SHIFT(kstate)&&!KEYMOD_CONTROL (kstate))
	{
		sprintf(SendCargHudMessage(),"%s - selected",hUcgo.ScnEditor_SelectNextCargoAvailableOnDisk());
		return 1;
	}
	// SHIFT+9 key "add last cargo selected by key 9"
	// If iSelectedCargo=-1 (default) add to the first free slot found
	if(key==OAPI_KEY_9&&KEYMOD_SHIFT(kstate)&&!KEYMOD_CONTROL (kstate))
	{
		if(hUcgo.ScnEditor_AddLastSelectedCargoToSlot(iSelectedCargo)==TRUE)
		{
			strcpy(SendCargHudMessage(),"Cargo added to ship");
		}
		else
		{
			if(iSelectedCargo<0)
			{
				strcpy(SendCargHudMessage(),"Cargo not added (ship full or weight excess ?)");
			}
			else
			{
				strcpy(SendCargHudMessage(),"Cargo not added (slot full ship full or weight excess ?)");
			}
		}
		return 1;
	}
	// "C" grapple cargo. If iSelectedCargo=-1 (default) add to the first free slot found
	if(key==OAPI_KEY_C&&!KEYMOD_SHIFT(kstate)&&!KEYMOD_CONTROL (kstate))
	{
		int ReturnedCode=hUcgo.GrappleOneCargo(iSelectedCargo);
		// for return code list see function "GrappleOneCargo" in the header
		switch(ReturnedCode)
		{
		case 1:
			strcpy(SendCargHudMessage(),"Cargo grappled");
			break;
		case 0:
			strcpy(SendCargHudMessage(),"No cargo in range");
			break;
		case -1:
			strcpy(SendCargHudMessage(),"cargo exceed maximum mass");
			break;
		case -2:
			strcpy(SendCargHudMessage(),"bad config, mesh not found or slot not declared");
			break;
		case -3:
			strcpy(SendCargHudMessage(),"Can't grapple cargo, slot not empty");
			break;
		case -4:
			strcpy(SendCargHudMessage(),"Can't grapple cargo,doors closed ");
			break;
		case -5:
			strcpy(SendCargHudMessage(),"Can't grapple cargo,Ship full");
			break;
		default:
			strcpy(SendCargHudMessage(),"Misc error Unable to grapple cargo");
		}
		return 1;
	}
	// SHIFT+C release cargo. If iSelectedCargo=-1 (default) release the first free slot found
	if(key==OAPI_KEY_C&&KEYMOD_SHIFT(kstate)&&!KEYMOD_CONTROL (kstate))
	{
		if(hUcgo.ReleaseOneCargo(iSelectedCargo)!=FALSE)
		{
			strcpy(SendCargHudMessage(),"Cargo released");
		}
		else
		{
			strcpy(SendCargHudMessage(),"Cargo not released (empty slot, no cargo aboard?)");
		}
		return 1;
	}
	// "8" show some info on cargo
	if(key==OAPI_KEY_8&&!KEYMOD_SHIFT(kstate)&&!KEYMOD_CONTROL (kstate))
	{
		sprintf(SendCargHudMessage(),"%i cargos aboard. "
		"Total cargos weight: %.0fkg",hUcgo.GetNbrCargoLoaded(),hUcgo.GetCargoTotalMass());
		return 1;
	}
	return 0;
}